# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_APPLY_TRANSFORMS_METADATA = Metadata(
    id="8209354b35fca6222f53525ec72fc9f4429a7c4a.boutiques",
    name="antsApplyTransforms",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsApplyTransformsWarpedOutputParameters = typing.TypedDict('AntsApplyTransformsWarpedOutputParameters', {
    "__STYX_TYPE__": typing.Literal["warpedOutput"],
    "warpedOutputFileName": str,
})


AntsApplyTransformsCompositeDisplacementFieldOutputParameters = typing.TypedDict('AntsApplyTransformsCompositeDisplacementFieldOutputParameters', {
    "__STYX_TYPE__": typing.Literal["compositeDisplacementFieldOutput"],
    "compositeDisplacementField": str,
    "printOutCompositeWarpFile": typing.NotRequired[typing.Literal[0, 1] | None],
})


AntsApplyTransformsGenericAffineTransformOutputParameters = typing.TypedDict('AntsApplyTransformsGenericAffineTransformOutputParameters', {
    "__STYX_TYPE__": typing.Literal["genericAffineTransformOutput"],
    "genericAffineTransformFile": str,
    "calculateInverse": typing.NotRequired[typing.Literal[0, 1] | None],
})


AntsApplyTransformsLinearParameters = typing.TypedDict('AntsApplyTransformsLinearParameters', {
    "__STYX_TYPE__": typing.Literal["linear"],
})


AntsApplyTransformsNearestNeighborParameters = typing.TypedDict('AntsApplyTransformsNearestNeighborParameters', {
    "__STYX_TYPE__": typing.Literal["nearestNeighbor"],
})


AntsApplyTransformsMultiLabelnoparamsParameters = typing.TypedDict('AntsApplyTransformsMultiLabelnoparamsParameters', {
    "__STYX_TYPE__": typing.Literal["multiLabelnoparams"],
})


AntsApplyTransformsSigmaParameters = typing.TypedDict('AntsApplyTransformsSigmaParameters', {
    "__STYX_TYPE__": typing.Literal["sigma"],
    "sigma": float,
})


AntsApplyTransformsAlphaParameters = typing.TypedDict('AntsApplyTransformsAlphaParameters', {
    "__STYX_TYPE__": typing.Literal["alpha"],
    "alpha": float,
})


AntsApplyTransformsParamParameters = typing.TypedDict('AntsApplyTransformsParamParameters', {
    "__STYX_TYPE__": typing.Literal["param"],
    "params": list[typing.Union[AntsApplyTransformsSigmaParameters, AntsApplyTransformsAlphaParameters]],
})


AntsApplyTransformsMultiLabelParameters = typing.TypedDict('AntsApplyTransformsMultiLabelParameters', {
    "__STYX_TYPE__": typing.Literal["multiLabel"],
    "params": AntsApplyTransformsParamParameters,
})


AntsApplyTransformsGaussianParameters = typing.TypedDict('AntsApplyTransformsGaussianParameters', {
    "__STYX_TYPE__": typing.Literal["gaussian"],
    "sigma": typing.NotRequired[float | None],
    "alpha": typing.NotRequired[float | None],
})


AntsApplyTransformsBsplineParameters = typing.TypedDict('AntsApplyTransformsBsplineParameters', {
    "__STYX_TYPE__": typing.Literal["bspline"],
    "order": typing.NotRequired[int | None],
})


AntsApplyTransformsCosineWindowedSincParameters = typing.TypedDict('AntsApplyTransformsCosineWindowedSincParameters', {
    "__STYX_TYPE__": typing.Literal["cosineWindowedSinc"],
})


AntsApplyTransformsWelchWindowedSincParameters = typing.TypedDict('AntsApplyTransformsWelchWindowedSincParameters', {
    "__STYX_TYPE__": typing.Literal["welchWindowedSinc"],
})


AntsApplyTransformsHammingWindowedSincParameters = typing.TypedDict('AntsApplyTransformsHammingWindowedSincParameters', {
    "__STYX_TYPE__": typing.Literal["hammingWindowedSinc"],
})


AntsApplyTransformsLanczosWindowedSincParameters = typing.TypedDict('AntsApplyTransformsLanczosWindowedSincParameters', {
    "__STYX_TYPE__": typing.Literal["lanczosWindowedSinc"],
})


AntsApplyTransformsGenericLabelParameters = typing.TypedDict('AntsApplyTransformsGenericLabelParameters', {
    "__STYX_TYPE__": typing.Literal["genericLabel"],
    "interpolator": typing.NotRequired[str | None],
})


AntsApplyTransformsTransformFileNameParameters = typing.TypedDict('AntsApplyTransformsTransformFileNameParameters', {
    "__STYX_TYPE__": typing.Literal["transformFileName"],
    "transformFileName": InputPathType,
})


AntsApplyTransformsUseInverseParameters = typing.TypedDict('AntsApplyTransformsUseInverseParameters', {
    "__STYX_TYPE__": typing.Literal["useInverse"],
    "transformFileName": InputPathType,
})


AntsApplyTransformsParameters = typing.TypedDict('AntsApplyTransformsParameters', {
    "__STYX_TYPE__": typing.Literal["antsApplyTransforms"],
    "dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "input_image_type": typing.NotRequired[typing.Literal[0, 1, 2, 3, 4, 5] | None],
    "input_image": InputPathType,
    "reference_image": InputPathType,
    "output": typing.Union[AntsApplyTransformsWarpedOutputParameters, AntsApplyTransformsCompositeDisplacementFieldOutputParameters, AntsApplyTransformsGenericAffineTransformOutputParameters],
    "interpolation": typing.NotRequired[typing.Union[AntsApplyTransformsLinearParameters, AntsApplyTransformsNearestNeighborParameters, AntsApplyTransformsMultiLabelnoparamsParameters, AntsApplyTransformsMultiLabelParameters, AntsApplyTransformsGaussianParameters, AntsApplyTransformsBsplineParameters, AntsApplyTransformsCosineWindowedSincParameters, AntsApplyTransformsWelchWindowedSincParameters, AntsApplyTransformsHammingWindowedSincParameters, AntsApplyTransformsLanczosWindowedSincParameters, AntsApplyTransformsGenericLabelParameters] | None],
    "output_data_type": typing.NotRequired[typing.Literal["char", "uchar", "short", "int", "float", "double", "default"] | None],
    "transform": typing.NotRequired[list[typing.Union[AntsApplyTransformsTransformFileNameParameters, AntsApplyTransformsUseInverseParameters]] | None],
    "default_value": typing.NotRequired[float | None],
    "static_cast_for_R": typing.NotRequired[str | None],
    "float": typing.NotRequired[typing.Literal[0, 1] | None],
    "verbose": typing.NotRequired[typing.Literal[0, 1] | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsApplyTransforms": ants_apply_transforms_cargs,
        "warpedOutput": ants_apply_transforms_warped_output_cargs,
        "compositeDisplacementFieldOutput": ants_apply_transforms_composite_displacement_field_output_cargs,
        "genericAffineTransformOutput": ants_apply_transforms_generic_affine_transform_output_cargs,
        "linear": ants_apply_transforms_linear_cargs,
        "nearestNeighbor": ants_apply_transforms_nearest_neighbor_cargs,
        "multiLabelnoparams": ants_apply_transforms_multi_labelnoparams_cargs,
        "multiLabel": ants_apply_transforms_multi_label_cargs,
        "param": ants_apply_transforms_param_cargs,
        "sigma": ants_apply_transforms_sigma_cargs,
        "alpha": ants_apply_transforms_alpha_cargs,
        "gaussian": ants_apply_transforms_gaussian_cargs,
        "bspline": ants_apply_transforms_bspline_cargs,
        "cosineWindowedSinc": ants_apply_transforms_cosine_windowed_sinc_cargs,
        "welchWindowedSinc": ants_apply_transforms_welch_windowed_sinc_cargs,
        "hammingWindowedSinc": ants_apply_transforms_hamming_windowed_sinc_cargs,
        "lanczosWindowedSinc": ants_apply_transforms_lanczos_windowed_sinc_cargs,
        "genericLabel": ants_apply_transforms_generic_label_cargs,
        "transformFileName": ants_apply_transforms_transform_file_name_cargs,
        "useInverse": ants_apply_transforms_use_inverse_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsApplyTransforms": ants_apply_transforms_outputs,
        "warpedOutput": ants_apply_transforms_warped_output_outputs,
        "compositeDisplacementFieldOutput": ants_apply_transforms_composite_displacement_field_output_outputs,
        "genericAffineTransformOutput": ants_apply_transforms_generic_affine_transform_output_outputs,
    }.get(t)


class AntsApplyTransformsWarpedOutputOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsApplyTransformsWarpedOutputParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_outfile: OutputPathType
    """Warped image."""


def ants_apply_transforms_warped_output_params(
    warped_output_file_name: str,
) -> AntsApplyTransformsWarpedOutputParameters:
    """
    Build parameters.
    
    Args:
        warped_output_file_name: Output file name.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "warpedOutput",
        "warpedOutputFileName": warped_output_file_name,
    }
    return params


def ants_apply_transforms_warped_output_cargs(
    params: AntsApplyTransformsWarpedOutputParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("warpedOutputFileName"))
    return cargs


def ants_apply_transforms_warped_output_outputs(
    params: AntsApplyTransformsWarpedOutputParameters,
    execution: Execution,
) -> AntsApplyTransformsWarpedOutputOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsApplyTransformsWarpedOutputOutputs(
        root=execution.output_file("."),
        output_image_outfile=execution.output_file(params.get("warpedOutputFileName")),
    )
    return ret


class AntsApplyTransformsCompositeDisplacementFieldOutputOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsApplyTransformsCompositeDisplacementFieldOutputParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_outfile: OutputPathType
    """Warped image."""


def ants_apply_transforms_composite_displacement_field_output_params(
    composite_displacement_field: str,
    print_out_composite_warp_file: typing.Literal[0, 1] | None = None,
) -> AntsApplyTransformsCompositeDisplacementFieldOutputParameters:
    """
    Build parameters.
    
    Args:
        composite_displacement_field: Output file name.
        print_out_composite_warp_file: Output a composite warp file instead of\
            a transformed image.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "compositeDisplacementFieldOutput",
        "compositeDisplacementField": composite_displacement_field,
    }
    if print_out_composite_warp_file is not None:
        params["printOutCompositeWarpFile"] = print_out_composite_warp_file
    return params


def ants_apply_transforms_composite_displacement_field_output_cargs(
    params: AntsApplyTransformsCompositeDisplacementFieldOutputParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("printOutCompositeWarpFile") is not None:
        cargs.append("[" + params.get("compositeDisplacementField") + ",printOutCompositeWarpFile=" + str(params.get("printOutCompositeWarpFile")) + "]")
    return cargs


def ants_apply_transforms_composite_displacement_field_output_outputs(
    params: AntsApplyTransformsCompositeDisplacementFieldOutputParameters,
    execution: Execution,
) -> AntsApplyTransformsCompositeDisplacementFieldOutputOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsApplyTransformsCompositeDisplacementFieldOutputOutputs(
        root=execution.output_file("."),
        output_image_outfile=execution.output_file(params.get("compositeDisplacementField")),
    )
    return ret


class AntsApplyTransformsGenericAffineTransformOutputOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AntsApplyTransformsGenericAffineTransformOutputParameters(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_outfile: OutputPathType
    """Warped image."""


def ants_apply_transforms_generic_affine_transform_output_params(
    generic_affine_transform_file: str,
    calculate_inverse: typing.Literal[0, 1] | None = None,
) -> AntsApplyTransformsGenericAffineTransformOutputParameters:
    """
    Build parameters.
    
    Args:
        generic_affine_transform_file: Output file name.
        calculate_inverse: Calculate the inverse of the affine transform.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "genericAffineTransformOutput",
        "genericAffineTransformFile": generic_affine_transform_file,
    }
    if calculate_inverse is not None:
        params["calculateInverse"] = calculate_inverse
    return params


def ants_apply_transforms_generic_affine_transform_output_cargs(
    params: AntsApplyTransformsGenericAffineTransformOutputParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("calculateInverse") is not None:
        cargs.append("Linear[" + params.get("genericAffineTransformFile") + ",calculateInverse=" + str(params.get("calculateInverse")) + "]")
    return cargs


def ants_apply_transforms_generic_affine_transform_output_outputs(
    params: AntsApplyTransformsGenericAffineTransformOutputParameters,
    execution: Execution,
) -> AntsApplyTransformsGenericAffineTransformOutputOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsApplyTransformsGenericAffineTransformOutputOutputs(
        root=execution.output_file("."),
        output_image_outfile=execution.output_file(params.get("genericAffineTransformFile")),
    )
    return ret


def ants_apply_transforms_linear_params(
) -> AntsApplyTransformsLinearParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "linear",
    }
    return params


def ants_apply_transforms_linear_cargs(
    params: AntsApplyTransformsLinearParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("Linear")
    return cargs


def ants_apply_transforms_nearest_neighbor_params(
) -> AntsApplyTransformsNearestNeighborParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "nearestNeighbor",
    }
    return params


def ants_apply_transforms_nearest_neighbor_cargs(
    params: AntsApplyTransformsNearestNeighborParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("NearestNeighbor")
    return cargs


def ants_apply_transforms_multi_labelnoparams_params(
) -> AntsApplyTransformsMultiLabelnoparamsParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "multiLabelnoparams",
    }
    return params


def ants_apply_transforms_multi_labelnoparams_cargs(
    params: AntsApplyTransformsMultiLabelnoparamsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("MultiLabel")
    return cargs


def ants_apply_transforms_sigma_params(
    sigma: float,
) -> AntsApplyTransformsSigmaParameters:
    """
    Build parameters.
    
    Args:
        sigma: Sigma value.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "sigma",
        "sigma": sigma,
    }
    return params


def ants_apply_transforms_sigma_cargs(
    params: AntsApplyTransformsSigmaParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("sigma=" + str(params.get("sigma")))
    return cargs


def ants_apply_transforms_alpha_params(
    alpha: float,
) -> AntsApplyTransformsAlphaParameters:
    """
    Build parameters.
    
    Args:
        alpha: Alpha value.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "alpha",
        "alpha": alpha,
    }
    return params


def ants_apply_transforms_alpha_cargs(
    params: AntsApplyTransformsAlphaParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("alpha=" + str(params.get("alpha")))
    return cargs


def ants_apply_transforms_param_params(
    params_: list[typing.Union[AntsApplyTransformsSigmaParameters, AntsApplyTransformsAlphaParameters]],
) -> AntsApplyTransformsParamParameters:
    """
    Build parameters.
    
    Args:
        params_:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "param",
        "params": params_,
    }
    return params


def ants_apply_transforms_param_cargs(
    params: AntsApplyTransformsParamParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + ",".join([a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("params")] for a in c]) + "]")
    return cargs


def ants_apply_transforms_multi_label_params(
    params_: AntsApplyTransformsParamParameters,
) -> AntsApplyTransformsMultiLabelParameters:
    """
    Build parameters.
    
    Args:
        params_:.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "multiLabel",
        "params": params_,
    }
    return params


def ants_apply_transforms_multi_label_cargs(
    params: AntsApplyTransformsMultiLabelParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("MultiLabel" + "".join(dyn_cargs(params.get("params")["__STYXTYPE__"])(params.get("params"), execution)))
    return cargs


def ants_apply_transforms_gaussian_params(
    sigma: float | None = None,
    alpha: float | None = None,
) -> AntsApplyTransformsGaussianParameters:
    """
    Build parameters.
    
    Args:
        sigma: Sigma value.
        alpha: Alpha value.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "gaussian",
    }
    if sigma is not None:
        params["sigma"] = sigma
    if alpha is not None:
        params["alpha"] = alpha
    return params


def ants_apply_transforms_gaussian_cargs(
    params: AntsApplyTransformsGaussianParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("sigma") is not None or params.get("alpha") is not None:
        cargs.append("Gaussian[sigma=" + (str(params.get("sigma")) if (params.get("sigma") is not None) else "") + ",alpha=" + (str(params.get("alpha")) if (params.get("alpha") is not None) else "") + "]")
    return cargs


def ants_apply_transforms_bspline_params(
    order: int | None = None,
) -> AntsApplyTransformsBsplineParameters:
    """
    Build parameters.
    
    Args:
        order: Order value.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "bspline",
    }
    if order is not None:
        params["order"] = order
    return params


def ants_apply_transforms_bspline_cargs(
    params: AntsApplyTransformsBsplineParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("order") is not None:
        cargs.append("BSpline[order=" + str(params.get("order")) + "]")
    return cargs


def ants_apply_transforms_cosine_windowed_sinc_params(
) -> AntsApplyTransformsCosineWindowedSincParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "cosineWindowedSinc",
    }
    return params


def ants_apply_transforms_cosine_windowed_sinc_cargs(
    params: AntsApplyTransformsCosineWindowedSincParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("CosineWindowedSinc")
    return cargs


def ants_apply_transforms_welch_windowed_sinc_params(
) -> AntsApplyTransformsWelchWindowedSincParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "welchWindowedSinc",
    }
    return params


def ants_apply_transforms_welch_windowed_sinc_cargs(
    params: AntsApplyTransformsWelchWindowedSincParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("WelchWindowedSinc")
    return cargs


def ants_apply_transforms_hamming_windowed_sinc_params(
) -> AntsApplyTransformsHammingWindowedSincParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "hammingWindowedSinc",
    }
    return params


def ants_apply_transforms_hamming_windowed_sinc_cargs(
    params: AntsApplyTransformsHammingWindowedSincParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("HammingWindowedSinc")
    return cargs


def ants_apply_transforms_lanczos_windowed_sinc_params(
) -> AntsApplyTransformsLanczosWindowedSincParameters:
    """
    Build parameters.
    
    Args:
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "lanczosWindowedSinc",
    }
    return params


def ants_apply_transforms_lanczos_windowed_sinc_cargs(
    params: AntsApplyTransformsLanczosWindowedSincParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("LanczosWindowedSinc")
    return cargs


def ants_apply_transforms_generic_label_params(
    interpolator: str | None = None,
) -> AntsApplyTransformsGenericLabelParameters:
    """
    Build parameters.
    
    Args:
        interpolator: Interpolator value.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "genericLabel",
    }
    if interpolator is not None:
        params["interpolator"] = interpolator
    return params


def ants_apply_transforms_generic_label_cargs(
    params: AntsApplyTransformsGenericLabelParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    if params.get("interpolator") is not None:
        cargs.append("GenericLabel[interpolator=" + params.get("interpolator") + "]")
    return cargs


def ants_apply_transforms_transform_file_name_params(
    transform_file_name: InputPathType,
) -> AntsApplyTransformsTransformFileNameParameters:
    """
    Build parameters.
    
    Args:
        transform_file_name: Transform file name.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "transformFileName",
        "transformFileName": transform_file_name,
    }
    return params


def ants_apply_transforms_transform_file_name_cargs(
    params: AntsApplyTransformsTransformFileNameParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("transformFileName")))
    return cargs


def ants_apply_transforms_use_inverse_params(
    transform_file_name: InputPathType,
) -> AntsApplyTransformsUseInverseParameters:
    """
    Build parameters.
    
    Args:
        transform_file_name: Transform file name.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "useInverse",
        "transformFileName": transform_file_name,
    }
    return params


def ants_apply_transforms_use_inverse_cargs(
    params: AntsApplyTransformsUseInverseParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("[" + execution.input_file(params.get("transformFileName")) + ",useInverse]")
    return cargs


class AntsApplyTransformsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_apply_transforms(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: typing.Union[AntsApplyTransformsWarpedOutputOutputs, AntsApplyTransformsCompositeDisplacementFieldOutputOutputs, AntsApplyTransformsGenericAffineTransformOutputOutputs]
    """Outputs from `AntsApplyTransformsWarpedOutputParameters` or
    `AntsApplyTransformsCompositeDisplacementFieldOutputParameters` or
    `AntsApplyTransformsGenericAffineTransformOutputParameters`."""


def ants_apply_transforms_params(
    input_image: InputPathType,
    reference_image: InputPathType,
    output: typing.Union[AntsApplyTransformsWarpedOutputParameters, AntsApplyTransformsCompositeDisplacementFieldOutputParameters, AntsApplyTransformsGenericAffineTransformOutputParameters],
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    input_image_type: typing.Literal[0, 1, 2, 3, 4, 5] | None = None,
    interpolation: typing.Union[AntsApplyTransformsLinearParameters, AntsApplyTransformsNearestNeighborParameters, AntsApplyTransformsMultiLabelnoparamsParameters, AntsApplyTransformsMultiLabelParameters, AntsApplyTransformsGaussianParameters, AntsApplyTransformsBsplineParameters, AntsApplyTransformsCosineWindowedSincParameters, AntsApplyTransformsWelchWindowedSincParameters, AntsApplyTransformsHammingWindowedSincParameters, AntsApplyTransformsLanczosWindowedSincParameters, AntsApplyTransformsGenericLabelParameters] | None = None,
    output_data_type: typing.Literal["char", "uchar", "short", "int", "float", "double", "default"] | None = None,
    transform: list[typing.Union[AntsApplyTransformsTransformFileNameParameters, AntsApplyTransformsUseInverseParameters]] | None = None,
    default_value: float | None = None,
    static_cast_for_r: str | None = None,
    float_: typing.Literal[0, 1] | None = None,
    verbose: typing.Literal[0, 1] | None = None,
) -> AntsApplyTransformsParameters:
    """
    Build parameters.
    
    Args:
        input_image: Currently, the only input objects supported are image\
            objects. However, the current framework allows for warping of other\
            objects such as meshes and point sets.
        reference_image: For warping input images, the reference image defines\
            the spacing, origin, size, and direction of the output warped image.
        output: One can either output the warped image or, if the boolean is\
            set, one can print out the displacement field based on the composite\
            transform and the reference image. A third option is to compose all\
            affine transforms and (if boolean is set) calculate its inverse which\
            is then written to an ITK file.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. if not specified, antswarp tries to infer\
            the dimensionality from the input image.
        input_image_type: Option specifying the input image type of scalar\
            (default), vector, tensor, time series, or multi-channel. A time series\
            image is a scalar image defined by an additional dimension for the time\
            component whereas a multi-channel image is a vector image with only\
            spatial dimensions. Five-dimensional images are e.g., AFNI stats image.
        interpolation: Several interpolation options are available in ITK.\
            These have all been made available.
        output_data_type: Output image data type. This is a direct typecast;\
            output values are not rescaled. Default is to use the internal data\
            type (float or double). uchar is unsigned char; others are signed.\
            WARNING: Outputs will be incorrect (overflowed/reinterpreted) if values\
            exceed the range allowed by your choice. Note that some pixel types are\
            not supported by some image formats. e.g. int is not supported by jpg.
        transform: Several transform options are supported including all those\
            defined in the ITK library in addition to a deformation field\
            transform. The ordering of the transformations follows the ordering\
            specified on the command line. An identity transform is pushed onto the\
            transformation stack. Each new transform encountered on the command\
            line is also pushed onto the transformation stack. Then, to warp the\
            input object, each point comprising the input object is warped first\
            according to the last transform pushed onto the stack followed by the\
            second to last transform, etc. until the last transform encountered\
            which is the identity transform. Also, it should be noted that the\
            inverse transform can be accommodated with the usual caveat that such\
            an inverse must be defined by the specified transform class.
        default_value: Default voxel value to be used with input images only.\
            Specifies the voxel value when the input point maps outside the output\
            domain. With tensor input images, specifies the default voxel\
            eigenvalues.
        static_cast_for_r: Forces static cast in ReadTransform (for R).
        float_: Use float instead of double for computations.
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsApplyTransforms",
        "input_image": input_image,
        "reference_image": reference_image,
        "output": output,
    }
    if dimensionality is not None:
        params["dimensionality"] = dimensionality
    if input_image_type is not None:
        params["input_image_type"] = input_image_type
    if interpolation is not None:
        params["interpolation"] = interpolation
    if output_data_type is not None:
        params["output_data_type"] = output_data_type
    if transform is not None:
        params["transform"] = transform
    if default_value is not None:
        params["default_value"] = default_value
    if static_cast_for_r is not None:
        params["static_cast_for_R"] = static_cast_for_r
    if float_ is not None:
        params["float"] = float_
    if verbose is not None:
        params["verbose"] = verbose
    return params


def ants_apply_transforms_cargs(
    params: AntsApplyTransformsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsApplyTransforms")
    if params.get("dimensionality") is not None:
        cargs.extend([
            "--dimensionality",
            str(params.get("dimensionality"))
        ])
    if params.get("input_image_type") is not None:
        cargs.extend([
            "--input-image-type",
            str(params.get("input_image_type"))
        ])
    cargs.extend([
        "--input",
        execution.input_file(params.get("input_image"))
    ])
    cargs.extend([
        "--reference-image",
        execution.input_file(params.get("reference_image"))
    ])
    cargs.extend([
        "--output",
        *dyn_cargs(params.get("output")["__STYXTYPE__"])(params.get("output"), execution)
    ])
    if params.get("interpolation") is not None:
        cargs.extend([
            "--interpolation",
            *dyn_cargs(params.get("interpolation")["__STYXTYPE__"])(params.get("interpolation"), execution)
        ])
    if params.get("output_data_type") is not None:
        cargs.extend([
            "--output-data-type",
            params.get("output_data_type")
        ])
    if params.get("transform") is not None:
        cargs.extend([
            "--transform",
            *[a for c in [dyn_cargs(s["__STYXTYPE__"])(s, execution) for s in params.get("transform")] for a in c]
        ])
    if params.get("default_value") is not None:
        cargs.extend([
            "--default-value",
            str(params.get("default_value"))
        ])
    if params.get("static_cast_for_R") is not None:
        cargs.extend([
            "--static-cast-for-R",
            params.get("static_cast_for_R")
        ])
    if params.get("float") is not None:
        cargs.extend([
            "--float",
            str(params.get("float"))
        ])
    if params.get("verbose") is not None:
        cargs.extend([
            "--verbose",
            str(params.get("verbose"))
        ])
    return cargs


def ants_apply_transforms_outputs(
    params: AntsApplyTransformsParameters,
    execution: Execution,
) -> AntsApplyTransformsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsApplyTransformsOutputs(
        root=execution.output_file("."),
        output=dyn_outputs(params.get("output")["__STYXTYPE__"])(params.get("output"), execution),
    )
    return ret


def ants_apply_transforms_execute(
    params: AntsApplyTransformsParameters,
    execution: Execution,
) -> AntsApplyTransformsOutputs:
    """
    antsApplyTransforms, applied to an input image, transforms it according to a
    reference image and a transform (or a set of transforms).
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsApplyTransformsOutputs`).
    """
    params = execution.params(params)
    cargs = ants_apply_transforms_cargs(params, execution)
    ret = ants_apply_transforms_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_apply_transforms(
    input_image: InputPathType,
    reference_image: InputPathType,
    output: typing.Union[AntsApplyTransformsWarpedOutputParameters, AntsApplyTransformsCompositeDisplacementFieldOutputParameters, AntsApplyTransformsGenericAffineTransformOutputParameters],
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    input_image_type: typing.Literal[0, 1, 2, 3, 4, 5] | None = None,
    interpolation: typing.Union[AntsApplyTransformsLinearParameters, AntsApplyTransformsNearestNeighborParameters, AntsApplyTransformsMultiLabelnoparamsParameters, AntsApplyTransformsMultiLabelParameters, AntsApplyTransformsGaussianParameters, AntsApplyTransformsBsplineParameters, AntsApplyTransformsCosineWindowedSincParameters, AntsApplyTransformsWelchWindowedSincParameters, AntsApplyTransformsHammingWindowedSincParameters, AntsApplyTransformsLanczosWindowedSincParameters, AntsApplyTransformsGenericLabelParameters] | None = None,
    output_data_type: typing.Literal["char", "uchar", "short", "int", "float", "double", "default"] | None = None,
    transform: list[typing.Union[AntsApplyTransformsTransformFileNameParameters, AntsApplyTransformsUseInverseParameters]] | None = None,
    default_value: float | None = None,
    static_cast_for_r: str | None = None,
    float_: typing.Literal[0, 1] | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> AntsApplyTransformsOutputs:
    """
    antsApplyTransforms, applied to an input image, transforms it according to a
    reference image and a transform (or a set of transforms).
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_image: Currently, the only input objects supported are image\
            objects. However, the current framework allows for warping of other\
            objects such as meshes and point sets.
        reference_image: For warping input images, the reference image defines\
            the spacing, origin, size, and direction of the output warped image.
        output: One can either output the warped image or, if the boolean is\
            set, one can print out the displacement field based on the composite\
            transform and the reference image. A third option is to compose all\
            affine transforms and (if boolean is set) calculate its inverse which\
            is then written to an ITK file.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. if not specified, antswarp tries to infer\
            the dimensionality from the input image.
        input_image_type: Option specifying the input image type of scalar\
            (default), vector, tensor, time series, or multi-channel. A time series\
            image is a scalar image defined by an additional dimension for the time\
            component whereas a multi-channel image is a vector image with only\
            spatial dimensions. Five-dimensional images are e.g., AFNI stats image.
        interpolation: Several interpolation options are available in ITK.\
            These have all been made available.
        output_data_type: Output image data type. This is a direct typecast;\
            output values are not rescaled. Default is to use the internal data\
            type (float or double). uchar is unsigned char; others are signed.\
            WARNING: Outputs will be incorrect (overflowed/reinterpreted) if values\
            exceed the range allowed by your choice. Note that some pixel types are\
            not supported by some image formats. e.g. int is not supported by jpg.
        transform: Several transform options are supported including all those\
            defined in the ITK library in addition to a deformation field\
            transform. The ordering of the transformations follows the ordering\
            specified on the command line. An identity transform is pushed onto the\
            transformation stack. Each new transform encountered on the command\
            line is also pushed onto the transformation stack. Then, to warp the\
            input object, each point comprising the input object is warped first\
            according to the last transform pushed onto the stack followed by the\
            second to last transform, etc. until the last transform encountered\
            which is the identity transform. Also, it should be noted that the\
            inverse transform can be accommodated with the usual caveat that such\
            an inverse must be defined by the specified transform class.
        default_value: Default voxel value to be used with input images only.\
            Specifies the voxel value when the input point maps outside the output\
            domain. With tensor input images, specifies the default voxel\
            eigenvalues.
        static_cast_for_r: Forces static cast in ReadTransform (for R).
        float_: Use float instead of double for computations.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsApplyTransformsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_APPLY_TRANSFORMS_METADATA)
    params = ants_apply_transforms_params(
        dimensionality=dimensionality,
        input_image_type=input_image_type,
        input_image=input_image,
        reference_image=reference_image,
        output=output,
        interpolation=interpolation,
        output_data_type=output_data_type,
        transform=transform,
        default_value=default_value,
        static_cast_for_r=static_cast_for_r,
        float_=float_,
        verbose=verbose,
    )
    return ants_apply_transforms_execute(params, execution)


__all__ = [
    "ANTS_APPLY_TRANSFORMS_METADATA",
    "AntsApplyTransformsAlphaParameters",
    "AntsApplyTransformsBsplineParameters",
    "AntsApplyTransformsCompositeDisplacementFieldOutputOutputs",
    "AntsApplyTransformsCompositeDisplacementFieldOutputParameters",
    "AntsApplyTransformsCosineWindowedSincParameters",
    "AntsApplyTransformsGaussianParameters",
    "AntsApplyTransformsGenericAffineTransformOutputOutputs",
    "AntsApplyTransformsGenericAffineTransformOutputParameters",
    "AntsApplyTransformsGenericLabelParameters",
    "AntsApplyTransformsHammingWindowedSincParameters",
    "AntsApplyTransformsLanczosWindowedSincParameters",
    "AntsApplyTransformsLinearParameters",
    "AntsApplyTransformsMultiLabelParameters",
    "AntsApplyTransformsMultiLabelnoparamsParameters",
    "AntsApplyTransformsNearestNeighborParameters",
    "AntsApplyTransformsOutputs",
    "AntsApplyTransformsParamParameters",
    "AntsApplyTransformsParameters",
    "AntsApplyTransformsSigmaParameters",
    "AntsApplyTransformsTransformFileNameParameters",
    "AntsApplyTransformsUseInverseParameters",
    "AntsApplyTransformsWarpedOutputOutputs",
    "AntsApplyTransformsWarpedOutputParameters",
    "AntsApplyTransformsWelchWindowedSincParameters",
    "ants_apply_transforms",
    "ants_apply_transforms_alpha_params",
    "ants_apply_transforms_bspline_params",
    "ants_apply_transforms_composite_displacement_field_output_params",
    "ants_apply_transforms_cosine_windowed_sinc_params",
    "ants_apply_transforms_gaussian_params",
    "ants_apply_transforms_generic_affine_transform_output_params",
    "ants_apply_transforms_generic_label_params",
    "ants_apply_transforms_hamming_windowed_sinc_params",
    "ants_apply_transforms_lanczos_windowed_sinc_params",
    "ants_apply_transforms_linear_params",
    "ants_apply_transforms_multi_label_params",
    "ants_apply_transforms_multi_labelnoparams_params",
    "ants_apply_transforms_nearest_neighbor_params",
    "ants_apply_transforms_param_params",
    "ants_apply_transforms_params",
    "ants_apply_transforms_sigma_params",
    "ants_apply_transforms_transform_file_name_params",
    "ants_apply_transforms_use_inverse_params",
    "ants_apply_transforms_warped_output_params",
    "ants_apply_transforms_welch_windowed_sinc_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TEXTURE_COOCCURRENCE_FEATURES_METADATA = Metadata(
    id="91591f77263330ed7a5bfa1c36e1edb089c3c166.boutiques",
    name="TextureCooccurrenceFeatures",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


TextureCooccurrenceFeaturesParameters = typing.TypedDict('TextureCooccurrenceFeaturesParameters', {
    "__STYX_TYPE__": typing.Literal["TextureCooccurrenceFeatures"],
    "image_dimension": int,
    "input_image": InputPathType,
    "number_of_bins_per_axis": typing.NotRequired[int | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "mask_label": typing.NotRequired[int | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "TextureCooccurrenceFeatures": texture_cooccurrence_features_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "TextureCooccurrenceFeatures": texture_cooccurrence_features_outputs,
    }.get(t)


class TextureCooccurrenceFeaturesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `texture_cooccurrence_features(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    features_output: OutputPathType
    """The output file containing the calculated texture co-occurrence
    features."""


def texture_cooccurrence_features_params(
    image_dimension: int,
    input_image: InputPathType,
    number_of_bins_per_axis: int | None = 256,
    mask_image: InputPathType | None = None,
    mask_label: int | None = 1,
) -> TextureCooccurrenceFeaturesParameters:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the input image, e.g., 2 for 2D\
            images, 3 for 3D images.
        input_image: The input image file for which texture co-occurrence\
            features will be calculated.
        number_of_bins_per_axis: The number of bins per axis to be used in the\
            histogram for texture calculation. Defaults to 256.
        mask_image: Optional mask image to specify the regions of interest in\
            the input image for which features will be calculated.
        mask_label: Label value in the mask image to specify which region to\
            process. Defaults to 1.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "TextureCooccurrenceFeatures",
        "image_dimension": image_dimension,
        "input_image": input_image,
    }
    if number_of_bins_per_axis is not None:
        params["number_of_bins_per_axis"] = number_of_bins_per_axis
    if mask_image is not None:
        params["mask_image"] = mask_image
    if mask_label is not None:
        params["mask_label"] = mask_label
    return params


def texture_cooccurrence_features_cargs(
    params: TextureCooccurrenceFeaturesParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("TextureCooccurrenceFeatures")
    cargs.append(str(params.get("image_dimension")))
    cargs.append(execution.input_file(params.get("input_image")))
    if params.get("number_of_bins_per_axis") is not None:
        cargs.append(str(params.get("number_of_bins_per_axis")))
    if params.get("mask_image") is not None:
        cargs.append(execution.input_file(params.get("mask_image")))
    if params.get("mask_label") is not None:
        cargs.append(str(params.get("mask_label")))
    return cargs


def texture_cooccurrence_features_outputs(
    params: TextureCooccurrenceFeaturesParameters,
    execution: Execution,
) -> TextureCooccurrenceFeaturesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TextureCooccurrenceFeaturesOutputs(
        root=execution.output_file("."),
        features_output=execution.output_file(pathlib.Path(params.get("input_image")).name + "_features.txt"),
    )
    return ret


def texture_cooccurrence_features_execute(
    params: TextureCooccurrenceFeaturesParameters,
    execution: Execution,
) -> TextureCooccurrenceFeaturesOutputs:
    """
    Calculates texture co-occurrence features such as Energy, Entropy, Inverse
    Difference Moment, Inertia, Cluster Shade, and Cluster Prominence from an input
    image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `TextureCooccurrenceFeaturesOutputs`).
    """
    params = execution.params(params)
    cargs = texture_cooccurrence_features_cargs(params, execution)
    ret = texture_cooccurrence_features_outputs(params, execution)
    execution.run(cargs)
    return ret


def texture_cooccurrence_features(
    image_dimension: int,
    input_image: InputPathType,
    number_of_bins_per_axis: int | None = 256,
    mask_image: InputPathType | None = None,
    mask_label: int | None = 1,
    runner: Runner | None = None,
) -> TextureCooccurrenceFeaturesOutputs:
    """
    Calculates texture co-occurrence features such as Energy, Entropy, Inverse
    Difference Moment, Inertia, Cluster Shade, and Cluster Prominence from an input
    image.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the input image, e.g., 2 for 2D\
            images, 3 for 3D images.
        input_image: The input image file for which texture co-occurrence\
            features will be calculated.
        number_of_bins_per_axis: The number of bins per axis to be used in the\
            histogram for texture calculation. Defaults to 256.
        mask_image: Optional mask image to specify the regions of interest in\
            the input image for which features will be calculated.
        mask_label: Label value in the mask image to specify which region to\
            process. Defaults to 1.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TextureCooccurrenceFeaturesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TEXTURE_COOCCURRENCE_FEATURES_METADATA)
    params = texture_cooccurrence_features_params(
        image_dimension=image_dimension,
        input_image=input_image,
        number_of_bins_per_axis=number_of_bins_per_axis,
        mask_image=mask_image,
        mask_label=mask_label,
    )
    return texture_cooccurrence_features_execute(params, execution)


__all__ = [
    "TEXTURE_COOCCURRENCE_FEATURES_METADATA",
    "TextureCooccurrenceFeaturesOutputs",
    "TextureCooccurrenceFeaturesParameters",
    "texture_cooccurrence_features",
    "texture_cooccurrence_features_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LESION_FILLING_METADATA = Metadata(
    id="e67424ecd9a0c5e13dd9b069bccb2e63e4348726.boutiques",
    name="LesionFilling",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


LesionFillingParameters = typing.TypedDict('LesionFillingParameters', {
    "__STYX_TYPE__": typing.Literal["LesionFilling"],
    "image_dimension": int,
    "t1_image": InputPathType,
    "lesion_mask": InputPathType,
    "output_lesion_filled": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "LesionFilling": lesion_filling_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "LesionFilling": lesion_filling_outputs,
    }.get(t)


class LesionFillingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `lesion_filling(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    lesion_filled_output: OutputPathType
    """Output image with filled lesions."""


def lesion_filling_params(
    image_dimension: int,
    t1_image: InputPathType,
    lesion_mask: InputPathType,
    output_lesion_filled: str,
) -> LesionFillingParameters:
    """
    Build parameters.
    
    Args:
        image_dimension: Dimensionality of the image (e.g., 2, 3).
        t1_image: Path to the T1 image file.
        lesion_mask: Path to the lesion mask image file.
        output_lesion_filled: Path for the output file with lesions filled.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "LesionFilling",
        "image_dimension": image_dimension,
        "t1_image": t1_image,
        "lesion_mask": lesion_mask,
        "output_lesion_filled": output_lesion_filled,
    }
    return params


def lesion_filling_cargs(
    params: LesionFillingParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("LesionFilling")
    cargs.append(str(params.get("image_dimension")))
    cargs.append(execution.input_file(params.get("t1_image")))
    cargs.append(execution.input_file(params.get("lesion_mask")))
    cargs.append(params.get("output_lesion_filled"))
    return cargs


def lesion_filling_outputs(
    params: LesionFillingParameters,
    execution: Execution,
) -> LesionFillingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LesionFillingOutputs(
        root=execution.output_file("."),
        lesion_filled_output=execution.output_file(params.get("output_lesion_filled")),
    )
    return ret


def lesion_filling_execute(
    params: LesionFillingParameters,
    execution: Execution,
) -> LesionFillingOutputs:
    """
    A tool for filling lesions in T1 images using a mask.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `LesionFillingOutputs`).
    """
    params = execution.params(params)
    cargs = lesion_filling_cargs(params, execution)
    ret = lesion_filling_outputs(params, execution)
    execution.run(cargs)
    return ret


def lesion_filling(
    image_dimension: int,
    t1_image: InputPathType,
    lesion_mask: InputPathType,
    output_lesion_filled: str,
    runner: Runner | None = None,
) -> LesionFillingOutputs:
    """
    A tool for filling lesions in T1 images using a mask.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: Dimensionality of the image (e.g., 2, 3).
        t1_image: Path to the T1 image file.
        lesion_mask: Path to the lesion mask image file.
        output_lesion_filled: Path for the output file with lesions filled.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LesionFillingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LESION_FILLING_METADATA)
    params = lesion_filling_params(
        image_dimension=image_dimension,
        t1_image=t1_image,
        lesion_mask=lesion_mask,
        output_lesion_filled=output_lesion_filled,
    )
    return lesion_filling_execute(params, execution)


__all__ = [
    "LESION_FILLING_METADATA",
    "LesionFillingOutputs",
    "LesionFillingParameters",
    "lesion_filling",
    "lesion_filling_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

IMAGE_SET_STATISTICS_METADATA = Metadata(
    id="9329b484e2f67dba69c548b9b9a373ea2a072aa8.boutiques",
    name="ImageSetStatistics",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


ImageSetStatisticsParameters = typing.TypedDict('ImageSetStatisticsParameters', {
    "__STYX_TYPE__": typing.Literal["ImageSetStatistics"],
    "image_dimension": int,
    "controls_list": InputPathType,
    "output_image": str,
    "which_stat": typing.Literal[0, 1, 2, 3, 4, 5, 6, 7],
    "roi": typing.NotRequired[InputPathType | None],
    "imagelist2": typing.NotRequired[InputPathType | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "ImageSetStatistics": image_set_statistics_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "ImageSetStatistics": image_set_statistics_outputs,
    }.get(t)


class ImageSetStatisticsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `image_set_statistics(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    computed_statistics_image: OutputPathType
    """The output image containing the computed statistics."""


def image_set_statistics_params(
    image_dimension: int,
    controls_list: InputPathType,
    output_image: str,
    which_stat: typing.Literal[0, 1, 2, 3, 4, 5, 6, 7],
    roi: InputPathType | None = None,
    imagelist2: InputPathType | None = None,
) -> ImageSetStatisticsParameters:
    """
    Build parameters.
    
    Args:
        image_dimension: The dimensionality of the images to be processed by\
            ImageSetStatistics.
        controls_list: Text file containing the list of control images.
        output_image: The output image file where the computed statistics are\
            stored.
        which_stat: Choice of statistic to compute: 0 for median, 1 for max\
            probability appearance, 2 for weighted mean appearance, 3 for trimmed\
            mean, 4 for max value, 5 for similarity-weighted (requires imagelist2),\
            6 for best local match label, 7 for max value from ROI.
        roi: Region of interest image file, optional depending on the whichstat\
            option.
        imagelist2: List of similarity images used for similarity-weighted\
            statistics. Required if whichstat equals 5 or 6.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "ImageSetStatistics",
        "image_dimension": image_dimension,
        "controls_list": controls_list,
        "output_image": output_image,
        "which_stat": which_stat,
    }
    if roi is not None:
        params["roi"] = roi
    if imagelist2 is not None:
        params["imagelist2"] = imagelist2
    return params


def image_set_statistics_cargs(
    params: ImageSetStatisticsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("ImageSetStatistics")
    cargs.append(str(params.get("image_dimension")))
    cargs.extend([
        "[CONTROLS_LIST]",
        execution.input_file(params.get("controls_list"))
    ])
    cargs.extend([
        "[OUTPUT_IMAGE]",
        params.get("output_image")
    ])
    cargs.extend([
        "[WHICH_STAT]",
        str(params.get("which_stat"))
    ])
    if params.get("roi") is not None:
        cargs.extend([
            "[ROI]",
            execution.input_file(params.get("roi"))
        ])
    if params.get("imagelist2") is not None:
        cargs.extend([
            "[IMAGELIST2]",
            execution.input_file(params.get("imagelist2"))
        ])
    return cargs


def image_set_statistics_outputs(
    params: ImageSetStatisticsParameters,
    execution: Execution,
) -> ImageSetStatisticsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ImageSetStatisticsOutputs(
        root=execution.output_file("."),
        computed_statistics_image=execution.output_file(params.get("output_image")),
    )
    return ret


def image_set_statistics_execute(
    params: ImageSetStatisticsParameters,
    execution: Execution,
) -> ImageSetStatisticsOutputs:
    """
    ImageSetStatistics computes statistics from a set of images. The whichstat
    option defines the type of statistic to compute, ranging from median to
    similarity-weighted metrics.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `ImageSetStatisticsOutputs`).
    """
    params = execution.params(params)
    cargs = image_set_statistics_cargs(params, execution)
    ret = image_set_statistics_outputs(params, execution)
    execution.run(cargs)
    return ret


def image_set_statistics(
    image_dimension: int,
    controls_list: InputPathType,
    output_image: str,
    which_stat: typing.Literal[0, 1, 2, 3, 4, 5, 6, 7],
    roi: InputPathType | None = None,
    imagelist2: InputPathType | None = None,
    runner: Runner | None = None,
) -> ImageSetStatisticsOutputs:
    """
    ImageSetStatistics computes statistics from a set of images. The whichstat
    option defines the type of statistic to compute, ranging from median to
    similarity-weighted metrics.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: The dimensionality of the images to be processed by\
            ImageSetStatistics.
        controls_list: Text file containing the list of control images.
        output_image: The output image file where the computed statistics are\
            stored.
        which_stat: Choice of statistic to compute: 0 for median, 1 for max\
            probability appearance, 2 for weighted mean appearance, 3 for trimmed\
            mean, 4 for max value, 5 for similarity-weighted (requires imagelist2),\
            6 for best local match label, 7 for max value from ROI.
        roi: Region of interest image file, optional depending on the whichstat\
            option.
        imagelist2: List of similarity images used for similarity-weighted\
            statistics. Required if whichstat equals 5 or 6.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ImageSetStatisticsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(IMAGE_SET_STATISTICS_METADATA)
    params = image_set_statistics_params(
        image_dimension=image_dimension,
        controls_list=controls_list,
        output_image=output_image,
        which_stat=which_stat,
        roi=roi,
        imagelist2=imagelist2,
    )
    return image_set_statistics_execute(params, execution)


__all__ = [
    "IMAGE_SET_STATISTICS_METADATA",
    "ImageSetStatisticsOutputs",
    "ImageSetStatisticsParameters",
    "image_set_statistics",
    "image_set_statistics_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

N3_BIAS_FIELD_CORRECTION_METADATA = Metadata(
    id="12e6ca015aa736902ad4369af67b842768a91549.boutiques",
    name="N3BiasFieldCorrection",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


N3BiasFieldCorrectionParameters = typing.TypedDict('N3BiasFieldCorrectionParameters', {
    "__STYX_TYPE__": typing.Literal["N3BiasFieldCorrection"],
    "image_dimensionality": typing.NotRequired[typing.Literal[2, 3, 4] | None],
    "input_image": InputPathType,
    "mask_image": typing.NotRequired[InputPathType | None],
    "rescale_intensities": typing.NotRequired[typing.Literal[0, 1] | None],
    "weight_image": typing.NotRequired[InputPathType | None],
    "shrink_factor": typing.NotRequired[int | None],
    "convergence": typing.NotRequired[str | None],
    "bspline_fitting": typing.NotRequired[str | None],
    "histogram_sharpening": typing.NotRequired[str | None],
    "output": str,
    "verbose": typing.NotRequired[typing.Literal[0, 1] | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "N3BiasFieldCorrection": n3_bias_field_correction_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "N3BiasFieldCorrection": n3_bias_field_correction_outputs,
    }.get(t)


class N3BiasFieldCorrectionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `n3_bias_field_correction(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    corrected_image: OutputPathType
    """The bias-corrected version of the input image."""
    bias_field: OutputPathType
    """The estimated bias field, if specified in the output."""


def n3_bias_field_correction_params(
    input_image: InputPathType,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    mask_image: InputPathType | None = None,
    rescale_intensities: typing.Literal[0, 1] | None = None,
    weight_image: InputPathType | None = None,
    shrink_factor: int | None = None,
    convergence: str | None = None,
    bspline_fitting: str | None = None,
    histogram_sharpening: str | None = None,
    verbose: typing.Literal[0, 1] | None = None,
) -> N3BiasFieldCorrectionParameters:
    """
    Build parameters.
    
    Args:
        input_image: A scalar image is expected as input for bias correction.\
            Since N3 log transforms the intensities, negative values or values\
            close to zero should be processed prior to correction.
        output: The bias-corrected version of the input image and optionally\
            the estimated bias field.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, N3 tries to infer the\
            dimensionality from the input image.
        mask_image: If a mask image is specified, the final bias correction is\
            only performed in the mask region. If a mask image is not specified,\
            the entire image region will be used as the mask region. Note: this\
            differs from the original N3 implementation.
        rescale_intensities: This option rescales the intensity range within\
            the user-specified mask to the original [min, max] range.
        weight_image: The weight image allows the user to perform a relative\
            weighting of specific voxels during the B-spline fitting.
        shrink_factor: Shrink factor to resample the input image. Commonly used\
            values are <= 4.
        convergence: Describes the convergence criteria with default value as\
            [50,0.0].
        bspline_fitting: Describes the parameters for B-Spline fitting.\
            Defaults are [splineDistance,4,3].
        histogram_sharpening: Describes histogram sharpening parameters;\
            defaults are [0.15,0.01,200].
        verbose: Verbose output.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "N3BiasFieldCorrection",
        "input_image": input_image,
        "output": output,
    }
    if image_dimensionality is not None:
        params["image_dimensionality"] = image_dimensionality
    if mask_image is not None:
        params["mask_image"] = mask_image
    if rescale_intensities is not None:
        params["rescale_intensities"] = rescale_intensities
    if weight_image is not None:
        params["weight_image"] = weight_image
    if shrink_factor is not None:
        params["shrink_factor"] = shrink_factor
    if convergence is not None:
        params["convergence"] = convergence
    if bspline_fitting is not None:
        params["bspline_fitting"] = bspline_fitting
    if histogram_sharpening is not None:
        params["histogram_sharpening"] = histogram_sharpening
    if verbose is not None:
        params["verbose"] = verbose
    return params


def n3_bias_field_correction_cargs(
    params: N3BiasFieldCorrectionParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("N3BiasFieldCorrection")
    if params.get("image_dimensionality") is not None:
        cargs.extend([
            "--image-dimensionality",
            str(params.get("image_dimensionality"))
        ])
    cargs.extend([
        "--input-image",
        execution.input_file(params.get("input_image"))
    ])
    if params.get("mask_image") is not None:
        cargs.extend([
            "--mask-image",
            execution.input_file(params.get("mask_image"))
        ])
    if params.get("rescale_intensities") is not None:
        cargs.extend([
            "--rescale-intensities",
            str(params.get("rescale_intensities"))
        ])
    if params.get("weight_image") is not None:
        cargs.extend([
            "--weight-image",
            execution.input_file(params.get("weight_image"))
        ])
    if params.get("shrink_factor") is not None:
        cargs.extend([
            "--shrink-factor",
            str(params.get("shrink_factor"))
        ])
    if params.get("convergence") is not None:
        cargs.extend([
            "--convergence",
            params.get("convergence")
        ])
    if params.get("bspline_fitting") is not None:
        cargs.extend([
            "--bspline-fitting",
            params.get("bspline_fitting")
        ])
    if params.get("histogram_sharpening") is not None:
        cargs.extend([
            "--histogram-sharpening",
            params.get("histogram_sharpening")
        ])
    cargs.extend([
        "--output",
        params.get("output")
    ])
    if params.get("verbose") is not None:
        cargs.extend([
            "--verbose",
            str(params.get("verbose"))
        ])
    return cargs


def n3_bias_field_correction_outputs(
    params: N3BiasFieldCorrectionParameters,
    execution: Execution,
) -> N3BiasFieldCorrectionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = N3BiasFieldCorrectionOutputs(
        root=execution.output_file("."),
        corrected_image=execution.output_file(params.get("output")),
        bias_field=execution.output_file("[BIS_FIELD]"),
    )
    return ret


def n3_bias_field_correction_execute(
    params: N3BiasFieldCorrectionParameters,
    execution: Execution,
) -> N3BiasFieldCorrectionOutputs:
    """
    This N3 is a variant of the popular N3 (nonparametric nonuniform normalization)
    retrospective bias correction algorithm. Based on the assumption that the
    corruption of the low frequency bias field can be modeled as a convolution of
    the intensity histogram by a Gaussian, the basic algorithmic protocol is to
    iterate between deconvolving the intensity histogram by a Gaussian, remapping
    the intensities, and then spatially smoothing this result by a B-spline modeling
    of the bias field itself.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `N3BiasFieldCorrectionOutputs`).
    """
    params = execution.params(params)
    cargs = n3_bias_field_correction_cargs(params, execution)
    ret = n3_bias_field_correction_outputs(params, execution)
    execution.run(cargs)
    return ret


def n3_bias_field_correction(
    input_image: InputPathType,
    output: str,
    image_dimensionality: typing.Literal[2, 3, 4] | None = None,
    mask_image: InputPathType | None = None,
    rescale_intensities: typing.Literal[0, 1] | None = None,
    weight_image: InputPathType | None = None,
    shrink_factor: int | None = None,
    convergence: str | None = None,
    bspline_fitting: str | None = None,
    histogram_sharpening: str | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> N3BiasFieldCorrectionOutputs:
    """
    This N3 is a variant of the popular N3 (nonparametric nonuniform normalization)
    retrospective bias correction algorithm. Based on the assumption that the
    corruption of the low frequency bias field can be modeled as a convolution of
    the intensity histogram by a Gaussian, the basic algorithmic protocol is to
    iterate between deconvolving the intensity histogram by a Gaussian, remapping
    the intensities, and then spatially smoothing this result by a B-spline modeling
    of the bias field itself.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        input_image: A scalar image is expected as input for bias correction.\
            Since N3 log transforms the intensities, negative values or values\
            close to zero should be processed prior to correction.
        output: The bias-corrected version of the input image and optionally\
            the estimated bias field.
        image_dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, N3 tries to infer the\
            dimensionality from the input image.
        mask_image: If a mask image is specified, the final bias correction is\
            only performed in the mask region. If a mask image is not specified,\
            the entire image region will be used as the mask region. Note: this\
            differs from the original N3 implementation.
        rescale_intensities: This option rescales the intensity range within\
            the user-specified mask to the original [min, max] range.
        weight_image: The weight image allows the user to perform a relative\
            weighting of specific voxels during the B-spline fitting.
        shrink_factor: Shrink factor to resample the input image. Commonly used\
            values are <= 4.
        convergence: Describes the convergence criteria with default value as\
            [50,0.0].
        bspline_fitting: Describes the parameters for B-Spline fitting.\
            Defaults are [splineDistance,4,3].
        histogram_sharpening: Describes histogram sharpening parameters;\
            defaults are [0.15,0.01,200].
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `N3BiasFieldCorrectionOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(N3_BIAS_FIELD_CORRECTION_METADATA)
    params = n3_bias_field_correction_params(
        image_dimensionality=image_dimensionality,
        input_image=input_image,
        mask_image=mask_image,
        rescale_intensities=rescale_intensities,
        weight_image=weight_image,
        shrink_factor=shrink_factor,
        convergence=convergence,
        bspline_fitting=bspline_fitting,
        histogram_sharpening=histogram_sharpening,
        output=output,
        verbose=verbose,
    )
    return n3_bias_field_correction_execute(params, execution)


__all__ = [
    "N3BiasFieldCorrectionOutputs",
    "N3BiasFieldCorrectionParameters",
    "N3_BIAS_FIELD_CORRECTION_METADATA",
    "n3_bias_field_correction",
    "n3_bias_field_correction_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANTS_CORTICAL_THICKNESS_SH_METADATA = Metadata(
    id="1af08fb76713108fbe119513e0ee5dea823c10df.boutiques",
    name="antsCorticalThickness.sh",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


AntsCorticalThicknessShParameters = typing.TypedDict('AntsCorticalThicknessShParameters', {
    "__STYX_TYPE__": typing.Literal["antsCorticalThickness.sh"],
    "image_dimension": typing.Literal[2, 3],
    "anatomical_image": InputPathType,
    "brain_template": InputPathType,
    "brain_extraction_probability_mask": InputPathType,
    "brain_segmentation_priors": str,
    "output_prefix": str,
    "image_file_suffix": typing.NotRequired[str | None],
    "template_for_t1_registration": typing.NotRequired[InputPathType | None],
    "extraction_registration_mask": typing.NotRequired[InputPathType | None],
    "keep_temporary_files": typing.NotRequired[typing.Literal[0, 1] | None],
    "denoise_anatomical_images": typing.NotRequired[typing.Literal[0, 1] | None],
    "max_iterations_for_registration": typing.NotRequired[str | None],
    "atropos_prior_segmentation_weight": typing.NotRequired[float | None],
    "number_of_segmentation_iterations": typing.NotRequired[int | None],
    "posterior_formulation": typing.NotRequired[str | None],
    "use_floating_point_precision": typing.NotRequired[typing.Literal[0, 1] | None],
    "use_random_seeding": typing.NotRequired[typing.Literal[0, 1] | None],
    "use_b_spline_smoothing": typing.NotRequired[typing.Literal[0, 1] | None],
    "cortical_thickness_prior_image": typing.NotRequired[InputPathType | None],
    "label_propagation": typing.NotRequired[str | None],
    "additional_priors_for_thickness": typing.NotRequired[str | None],
    "use_quick_registration_parameters": typing.NotRequired[typing.Literal[0, 1] | None],
    "atropos_iterations": typing.NotRequired[int | None],
    "script_stage_to_run": typing.NotRequired[int | None],
    "test_debug_mode": typing.NotRequired[int | None],
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "antsCorticalThickness.sh": ants_cortical_thickness_sh_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "antsCorticalThickness.sh": ants_cortical_thickness_sh_outputs,
    }.get(t)


class AntsCorticalThicknessShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ants_cortical_thickness_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cortical_thickness: OutputPathType
    """The output cortical thickness map."""
    brain_extraction_mask: OutputPathType
    """The brain extraction mask."""
    brain_segmentation: OutputPathType
    """The brain segmentation image."""
    segmentation_posteriors: OutputPathType
    """The segmentation posteriors for different tissue classes."""


def ants_cortical_thickness_sh_params(
    image_dimension: typing.Literal[2, 3],
    anatomical_image: InputPathType,
    brain_template: InputPathType,
    brain_extraction_probability_mask: InputPathType,
    brain_segmentation_priors: str,
    output_prefix: str,
    image_file_suffix: str | None = None,
    template_for_t1_registration: InputPathType | None = None,
    extraction_registration_mask: InputPathType | None = None,
    keep_temporary_files: typing.Literal[0, 1] | None = None,
    denoise_anatomical_images: typing.Literal[0, 1] | None = None,
    max_iterations_for_registration: str | None = None,
    atropos_prior_segmentation_weight: float | None = None,
    number_of_segmentation_iterations: int | None = None,
    posterior_formulation: str | None = None,
    use_floating_point_precision: typing.Literal[0, 1] | None = None,
    use_random_seeding: typing.Literal[0, 1] | None = None,
    use_b_spline_smoothing: typing.Literal[0, 1] | None = None,
    cortical_thickness_prior_image: InputPathType | None = None,
    label_propagation: str | None = None,
    additional_priors_for_thickness: str | None = None,
    use_quick_registration_parameters: typing.Literal[0, 1] | None = None,
    atropos_iterations: int | None = None,
    script_stage_to_run: int | None = None,
    test_debug_mode: int | None = None,
) -> AntsCorticalThicknessShParameters:
    """
    Build parameters.
    
    Args:
        image_dimension: 2 or 3 for 2- or 3-dimensional image.
        anatomical_image: Structural intensity image, typically T1.
        brain_template: Anatomical intensity template. This template is not\
            skull-stripped.
        brain_extraction_probability_mask: Brain probability mask in the\
            segmentation template space. A binary mask is acceptable.
        brain_segmentation_priors: Tissue probability priors corresponding to\
            the template image specified with the -e option. At least four priors\
            must exist, corresponding to CSF, Cortical GM, WM, Subcortical GM.
        output_prefix: Output prefix for the generated filenames.
        image_file_suffix: Any of the standard ITK IO formats e.g. nrrd, nii.gz\
            (default), mhd.
        template_for_t1_registration: Anatomical intensity template. This\
            template must be skull-stripped.
        extraction_registration_mask: Binary metric mask defined in the\
            segmentation template space (-e). Only used in brain extraction\
            registration.
        keep_temporary_files: Keep brain extraction/segmentation warps, etc\
            (default = 0).
        denoise_anatomical_images: Denoise anatomical images (default = 0).
        max_iterations_for_registration: ANTS registration max iterations\
            (default = 100x100x70x20).
        atropos_prior_segmentation_weight: Atropos spatial prior probability\
            weight for the segmentation (default = 0.25).
        number_of_segmentation_iterations: N4 -> Atropos -> N4 iterations\
            during segmentation (default = 3).
        posterior_formulation: Atropos posterior formulation and whether or not\
            to use mixture model proportions. e.g 'Socrates[ 1 ]' (default) or\
            'Aristotle[ 1 ]'.
        use_floating_point_precision: Use single float precision in\
            registrations (default = 0).
        use_random_seeding: Use random number generated from system clock\
            (default = 1).
        use_b_spline_smoothing: Use B-spline SyN for registrations and B-spline\
            exponential mapping in DiReCT (default = 0).
        cortical_thickness_prior_image: Cortical thickness prior image in the\
            template space, with an estimated upper limit of cortical thickness at\
            each voxel.
        label_propagation: Incorporate a distance prior on the posterior\
            formulation.
        additional_priors_for_thickness: Add segmentation classes for thickness\
            estimation.
        use_quick_registration_parameters: Use antsRegistrationSyNQuick.sh for\
            registrations (default = 0).
        atropos_iterations: Number of iterations within Atropos (default = 5).
        script_stage_to_run: Which stage of ACT to run (default = 0, run all).
        test_debug_mode: If > 0, runs a faster version of the script. Only for\
            testing (default = 0).
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "antsCorticalThickness.sh",
        "image_dimension": image_dimension,
        "anatomical_image": anatomical_image,
        "brain_template": brain_template,
        "brain_extraction_probability_mask": brain_extraction_probability_mask,
        "brain_segmentation_priors": brain_segmentation_priors,
        "output_prefix": output_prefix,
    }
    if image_file_suffix is not None:
        params["image_file_suffix"] = image_file_suffix
    if template_for_t1_registration is not None:
        params["template_for_t1_registration"] = template_for_t1_registration
    if extraction_registration_mask is not None:
        params["extraction_registration_mask"] = extraction_registration_mask
    if keep_temporary_files is not None:
        params["keep_temporary_files"] = keep_temporary_files
    if denoise_anatomical_images is not None:
        params["denoise_anatomical_images"] = denoise_anatomical_images
    if max_iterations_for_registration is not None:
        params["max_iterations_for_registration"] = max_iterations_for_registration
    if atropos_prior_segmentation_weight is not None:
        params["atropos_prior_segmentation_weight"] = atropos_prior_segmentation_weight
    if number_of_segmentation_iterations is not None:
        params["number_of_segmentation_iterations"] = number_of_segmentation_iterations
    if posterior_formulation is not None:
        params["posterior_formulation"] = posterior_formulation
    if use_floating_point_precision is not None:
        params["use_floating_point_precision"] = use_floating_point_precision
    if use_random_seeding is not None:
        params["use_random_seeding"] = use_random_seeding
    if use_b_spline_smoothing is not None:
        params["use_b_spline_smoothing"] = use_b_spline_smoothing
    if cortical_thickness_prior_image is not None:
        params["cortical_thickness_prior_image"] = cortical_thickness_prior_image
    if label_propagation is not None:
        params["label_propagation"] = label_propagation
    if additional_priors_for_thickness is not None:
        params["additional_priors_for_thickness"] = additional_priors_for_thickness
    if use_quick_registration_parameters is not None:
        params["use_quick_registration_parameters"] = use_quick_registration_parameters
    if atropos_iterations is not None:
        params["atropos_iterations"] = atropos_iterations
    if script_stage_to_run is not None:
        params["script_stage_to_run"] = script_stage_to_run
    if test_debug_mode is not None:
        params["test_debug_mode"] = test_debug_mode
    return params


def ants_cortical_thickness_sh_cargs(
    params: AntsCorticalThicknessShParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("antsCorticalThickness.sh")
    cargs.extend([
        "-d",
        str(params.get("image_dimension"))
    ])
    cargs.extend([
        "-a",
        execution.input_file(params.get("anatomical_image"))
    ])
    cargs.extend([
        "-e",
        execution.input_file(params.get("brain_template"))
    ])
    cargs.extend([
        "-m",
        execution.input_file(params.get("brain_extraction_probability_mask"))
    ])
    cargs.extend([
        "-p",
        params.get("brain_segmentation_priors")
    ])
    cargs.extend([
        "-o",
        params.get("output_prefix")
    ])
    if params.get("image_file_suffix") is not None:
        cargs.extend([
            "-s",
            params.get("image_file_suffix")
        ])
    if params.get("template_for_t1_registration") is not None:
        cargs.extend([
            "-t",
            execution.input_file(params.get("template_for_t1_registration"))
        ])
    if params.get("extraction_registration_mask") is not None:
        cargs.extend([
            "-f",
            execution.input_file(params.get("extraction_registration_mask"))
        ])
    if params.get("keep_temporary_files") is not None:
        cargs.extend([
            "-k",
            str(params.get("keep_temporary_files"))
        ])
    if params.get("denoise_anatomical_images") is not None:
        cargs.extend([
            "-g",
            str(params.get("denoise_anatomical_images"))
        ])
    if params.get("max_iterations_for_registration") is not None:
        cargs.extend([
            "-i",
            params.get("max_iterations_for_registration")
        ])
    if params.get("atropos_prior_segmentation_weight") is not None:
        cargs.extend([
            "-w",
            str(params.get("atropos_prior_segmentation_weight"))
        ])
    if params.get("number_of_segmentation_iterations") is not None:
        cargs.extend([
            "-n",
            str(params.get("number_of_segmentation_iterations"))
        ])
    if params.get("posterior_formulation") is not None:
        cargs.extend([
            "-b",
            params.get("posterior_formulation")
        ])
    if params.get("use_floating_point_precision") is not None:
        cargs.extend([
            "-j",
            str(params.get("use_floating_point_precision"))
        ])
    if params.get("use_random_seeding") is not None:
        cargs.extend([
            "-u",
            str(params.get("use_random_seeding"))
        ])
    if params.get("use_b_spline_smoothing") is not None:
        cargs.extend([
            "-v",
            str(params.get("use_b_spline_smoothing"))
        ])
    if params.get("cortical_thickness_prior_image") is not None:
        cargs.extend([
            "-r",
            execution.input_file(params.get("cortical_thickness_prior_image"))
        ])
    if params.get("label_propagation") is not None:
        cargs.extend([
            "-l",
            params.get("label_propagation")
        ])
    if params.get("additional_priors_for_thickness") is not None:
        cargs.extend([
            "-c",
            params.get("additional_priors_for_thickness")
        ])
    if params.get("use_quick_registration_parameters") is not None:
        cargs.extend([
            "-q",
            str(params.get("use_quick_registration_parameters"))
        ])
    if params.get("atropos_iterations") is not None:
        cargs.extend([
            "-x",
            str(params.get("atropos_iterations"))
        ])
    if params.get("script_stage_to_run") is not None:
        cargs.extend([
            "-y",
            str(params.get("script_stage_to_run"))
        ])
    if params.get("test_debug_mode") is not None:
        cargs.extend([
            "-z",
            str(params.get("test_debug_mode"))
        ])
    return cargs


def ants_cortical_thickness_sh_outputs(
    params: AntsCorticalThicknessShParameters,
    execution: Execution,
) -> AntsCorticalThicknessShOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AntsCorticalThicknessShOutputs(
        root=execution.output_file("."),
        cortical_thickness=execution.output_file(params.get("output_prefix") + "CorticalThickness.nii.gz"),
        brain_extraction_mask=execution.output_file(params.get("output_prefix") + "BrainExtractionMask.nii.gz"),
        brain_segmentation=execution.output_file(params.get("output_prefix") + "BrainSegmentation.nii.gz"),
        segmentation_posteriors=execution.output_file(params.get("output_prefix") + "BrainSegmentationPosteriors*.nii.gz"),
    )
    return ret


def ants_cortical_thickness_sh_execute(
    params: AntsCorticalThicknessShParameters,
    execution: Execution,
) -> AntsCorticalThicknessShOutputs:
    """
    This script performs T1 anatomical brain processing including brain extraction,
    brain n-tissue segmentation, cortical thickness estimation, and optional
    registration to a template.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `AntsCorticalThicknessShOutputs`).
    """
    params = execution.params(params)
    cargs = ants_cortical_thickness_sh_cargs(params, execution)
    ret = ants_cortical_thickness_sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def ants_cortical_thickness_sh(
    image_dimension: typing.Literal[2, 3],
    anatomical_image: InputPathType,
    brain_template: InputPathType,
    brain_extraction_probability_mask: InputPathType,
    brain_segmentation_priors: str,
    output_prefix: str,
    image_file_suffix: str | None = None,
    template_for_t1_registration: InputPathType | None = None,
    extraction_registration_mask: InputPathType | None = None,
    keep_temporary_files: typing.Literal[0, 1] | None = None,
    denoise_anatomical_images: typing.Literal[0, 1] | None = None,
    max_iterations_for_registration: str | None = None,
    atropos_prior_segmentation_weight: float | None = None,
    number_of_segmentation_iterations: int | None = None,
    posterior_formulation: str | None = None,
    use_floating_point_precision: typing.Literal[0, 1] | None = None,
    use_random_seeding: typing.Literal[0, 1] | None = None,
    use_b_spline_smoothing: typing.Literal[0, 1] | None = None,
    cortical_thickness_prior_image: InputPathType | None = None,
    label_propagation: str | None = None,
    additional_priors_for_thickness: str | None = None,
    use_quick_registration_parameters: typing.Literal[0, 1] | None = None,
    atropos_iterations: int | None = None,
    script_stage_to_run: int | None = None,
    test_debug_mode: int | None = None,
    runner: Runner | None = None,
) -> AntsCorticalThicknessShOutputs:
    """
    This script performs T1 anatomical brain processing including brain extraction,
    brain n-tissue segmentation, cortical thickness estimation, and optional
    registration to a template.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        image_dimension: 2 or 3 for 2- or 3-dimensional image.
        anatomical_image: Structural intensity image, typically T1.
        brain_template: Anatomical intensity template. This template is not\
            skull-stripped.
        brain_extraction_probability_mask: Brain probability mask in the\
            segmentation template space. A binary mask is acceptable.
        brain_segmentation_priors: Tissue probability priors corresponding to\
            the template image specified with the -e option. At least four priors\
            must exist, corresponding to CSF, Cortical GM, WM, Subcortical GM.
        output_prefix: Output prefix for the generated filenames.
        image_file_suffix: Any of the standard ITK IO formats e.g. nrrd, nii.gz\
            (default), mhd.
        template_for_t1_registration: Anatomical intensity template. This\
            template must be skull-stripped.
        extraction_registration_mask: Binary metric mask defined in the\
            segmentation template space (-e). Only used in brain extraction\
            registration.
        keep_temporary_files: Keep brain extraction/segmentation warps, etc\
            (default = 0).
        denoise_anatomical_images: Denoise anatomical images (default = 0).
        max_iterations_for_registration: ANTS registration max iterations\
            (default = 100x100x70x20).
        atropos_prior_segmentation_weight: Atropos spatial prior probability\
            weight for the segmentation (default = 0.25).
        number_of_segmentation_iterations: N4 -> Atropos -> N4 iterations\
            during segmentation (default = 3).
        posterior_formulation: Atropos posterior formulation and whether or not\
            to use mixture model proportions. e.g 'Socrates[ 1 ]' (default) or\
            'Aristotle[ 1 ]'.
        use_floating_point_precision: Use single float precision in\
            registrations (default = 0).
        use_random_seeding: Use random number generated from system clock\
            (default = 1).
        use_b_spline_smoothing: Use B-spline SyN for registrations and B-spline\
            exponential mapping in DiReCT (default = 0).
        cortical_thickness_prior_image: Cortical thickness prior image in the\
            template space, with an estimated upper limit of cortical thickness at\
            each voxel.
        label_propagation: Incorporate a distance prior on the posterior\
            formulation.
        additional_priors_for_thickness: Add segmentation classes for thickness\
            estimation.
        use_quick_registration_parameters: Use antsRegistrationSyNQuick.sh for\
            registrations (default = 0).
        atropos_iterations: Number of iterations within Atropos (default = 5).
        script_stage_to_run: Which stage of ACT to run (default = 0, run all).
        test_debug_mode: If > 0, runs a faster version of the script. Only for\
            testing (default = 0).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AntsCorticalThicknessShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANTS_CORTICAL_THICKNESS_SH_METADATA)
    params = ants_cortical_thickness_sh_params(
        image_dimension=image_dimension,
        anatomical_image=anatomical_image,
        brain_template=brain_template,
        brain_extraction_probability_mask=brain_extraction_probability_mask,
        brain_segmentation_priors=brain_segmentation_priors,
        output_prefix=output_prefix,
        image_file_suffix=image_file_suffix,
        template_for_t1_registration=template_for_t1_registration,
        extraction_registration_mask=extraction_registration_mask,
        keep_temporary_files=keep_temporary_files,
        denoise_anatomical_images=denoise_anatomical_images,
        max_iterations_for_registration=max_iterations_for_registration,
        atropos_prior_segmentation_weight=atropos_prior_segmentation_weight,
        number_of_segmentation_iterations=number_of_segmentation_iterations,
        posterior_formulation=posterior_formulation,
        use_floating_point_precision=use_floating_point_precision,
        use_random_seeding=use_random_seeding,
        use_b_spline_smoothing=use_b_spline_smoothing,
        cortical_thickness_prior_image=cortical_thickness_prior_image,
        label_propagation=label_propagation,
        additional_priors_for_thickness=additional_priors_for_thickness,
        use_quick_registration_parameters=use_quick_registration_parameters,
        atropos_iterations=atropos_iterations,
        script_stage_to_run=script_stage_to_run,
        test_debug_mode=test_debug_mode,
    )
    return ants_cortical_thickness_sh_execute(params, execution)


__all__ = [
    "ANTS_CORTICAL_THICKNESS_SH_METADATA",
    "AntsCorticalThicknessShOutputs",
    "AntsCorticalThicknessShParameters",
    "ants_cortical_thickness_sh",
    "ants_cortical_thickness_sh_params",
]
